@@ -28,6 +28,7 @@ struct ssh_auth_request {
     int method;
     char *password;
     struct ssh_key_struct *pubkey;
+    struct ssh_key_struct *server_pubkey;
     char *sigtype;
     enum ssh_publickey_state_e signature_state;
     char kbdint_response;
@@ -92,6 +93,10 @@ struct ssh_message_struct {
     struct ssh_global_request global_request;
 };
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 SSH_PACKET_CALLBACK(ssh_packet_channel_open);
 SSH_PACKET_CALLBACK(ssh_packet_global_request);
 
@@ -104,4 +109,8 @@ int ssh_message_handle_channel_request(ssh_session session, ssh_channel channel,
     const char *request, uint8_t want_reply);
 ssh_message ssh_message_pop_head(ssh_session session);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* MESSAGES_H_ */
