@@ -30,6 +30,7 @@
 #include "libssh/priv.h"
 #include "libssh/scp.h"
 #include "libssh/misc.h"
+#include "libssh/session.h"
 
 /**
  * @defgroup libssh_scp The SSH scp functions
@@ -197,6 +198,17 @@ int ssh_scp_init(ssh_scp scp)
         return SSH_ERROR;
     }
 
+    /* Some servers do not handle the quoting well. Pass in the raw file
+     * location */
+    if (scp->session->flags & SSH_SESSION_FLAG_SCP_QUOTING_BROKEN) {
+        free(quoted_location);
+        quoted_location = strdup(scp->location);
+        if (quoted_location == NULL) {
+            ssh_set_error_oom(scp->session);
+            return SSH_ERROR;
+        }
+    }
+
     if (scp->mode == SSH_SCP_WRITE) {
         snprintf(execbuffer, sizeof(execbuffer), "scp -t %s %s",
                 scp->recursive ? "-r" : "", quoted_location);
