@@ -31,11 +31,17 @@
 #include <errno.h>
 #include <pwd.h>
 
-#include "torture.h"
-#include "torture_key.h"
+#ifdef HAVE_VALGRIND_VALGRIND_H
+#include <valgrind/valgrind.h>
+#endif
+
+#include "libssh/buffer.h"
 #include "libssh/libssh.h"
 #include "libssh/priv.h"
 #include "libssh/session.h"
+#include "libssh/sftp_priv.h"
+#include "torture.h"
+#include "torture_key.h"
 
 #include "test_server.h"
 #include "default_cb.h"
@@ -504,10 +510,10 @@ static void torture_server_test_sftp_function(void **state)
 static void torture_server_sftp_open_read_write(void **state)
 {
     struct test_server_st *tss = *state;
-    struct torture_state *s;
-    struct torture_sftp *tsftp;
-    sftp_session sftp;
-    ssh_session session;
+    struct torture_state *s = NULL;
+    struct torture_sftp *tsftp = NULL;
+    sftp_session sftp = NULL;
+    ssh_session session = NULL;
     sftp_attributes a = NULL;
     sftp_file new_file = NULL;
     char tmp_file[PATH_MAX] = {0};
@@ -563,6 +569,30 @@ static void torture_server_sftp_open_read_write(void **state)
     assert_int_equal(a->type, SSH_FILEXFER_TYPE_REGULAR);
     sftp_attributes_free(a);
 
+    /*
+     * Now that file exists and contains some data, lets try O_TRUNC,
+     * mode is ignored
+     */
+    new_file = sftp_open(sftp, tmp_file, O_WRONLY | O_TRUNC, 0);
+    assert_non_null(new_file);
+
+    /* Verify that the existing data in the file has been truncated */
+    a = sftp_stat(sftp, tmp_file);
+    assert_non_null(a);
+    assert_int_equal(a->size, 0); /* No content due to truncation */
+    sftp_attributes_free(a);
+
+    /* Write should work ok */
+    write_len = sftp_write(new_file, data, sizeof(data));
+    assert_int_equal(write_len, sizeof(data));
+
+    /* Reading should fail */
+    read_len = sftp_read(new_file, read_data, sizeof(read_data));
+    assert_int_equal(read_len, SSH_ERROR);
+
+    rc = sftp_close(new_file);
+    assert_ssh_return_code(session, rc);
+
     /*
      * Now, lets try O_APPEND, mode is ignored
      */
@@ -1100,6 +1130,135 @@ static void torture_server_sftp_handles_exhaustion(void **state)
     }
 }
 
+static void torture_server_sftp_handle_overrun(void **state)
+{
+    struct test_server_st *tss = *state;
+    struct torture_state *s = NULL;
+    struct torture_sftp *tsftp = NULL;
+    char name[128] = {0};
+    sftp_session sftp = NULL;
+    sftp_file handle = NULL;
+    ssh_buffer buffer = NULL;
+    uint32_t id, bad_handle = SFTP_HANDLES, bad_handle_len = 4;
+    sftp_message msg = NULL;
+    sftp_status_message status = NULL;
+    int rc;
+
+    assert_non_null(tss);
+
+    s = tss->state;
+    assert_non_null(s);
+
+    tsftp = s->ssh.tsftp;
+    assert_non_null(tsftp);
+
+    sftp = tsftp->sftp;
+    assert_non_null(sftp);
+
+    /* Initialize the sftp handles by opening first file */
+    snprintf(name, sizeof(name), "%s/file", tsftp->testdir);
+    handle = sftp_open(sftp, name, O_WRONLY | O_CREAT, 0700);
+    assert_non_null(handle);
+
+    /* Craft an malicious SFTP packet trying to access handle 256
+     * (SFTP_HANDLES) */
+    buffer = ssh_buffer_new();
+    id = sftp_get_new_id(sftp);
+    assert_non_null(buffer);
+
+    rc = ssh_buffer_pack(buffer,
+                         "ddPqd",
+                         id,
+                         (uint32_t)bad_handle_len,
+                         (size_t)bad_handle_len,
+                         &bad_handle, /* a 32b int as ssh_string */
+                         (uint64_t)0,
+                         (uint32_t)1024);
+    assert_int_equal(rc, SSH_OK);
+    rc = sftp_packet_write(sftp, SSH_FXP_READ, buffer);
+    SSH_BUFFER_FREE(buffer);
+    assert_int_equal(rc, 29);
+
+    rc = sftp_recv_response_msg(sftp, id, true, &msg);
+    assert_int_equal(rc, SSH_OK);
+    assert_int_equal(msg->packet_type, SSH_FXP_STATUS);
+    status = parse_status_msg(msg);
+    sftp_message_free(msg);
+    assert_int_equal(status->status, SSH_FX_INVALID_HANDLE);
+    status_msg_free(status);
+
+    rc = sftp_close(handle);
+    assert_int_equal(rc, SSH_OK);
+}
+
+static void torture_server_sftp_payload_overrun(void **state)
+{
+    struct test_server_st *tss = *state;
+    struct torture_state *s = NULL;
+    struct torture_sftp *tsftp = NULL;
+    char name[128] = {0};
+    sftp_session sftp = NULL;
+    sftp_file handle = NULL;
+    ssh_buffer buffer = NULL;
+    uint32_t id, bad_payload_len = 0x7ffffffc;
+    int rc;
+
+#ifdef HAVE_VALGRIND_VALGRIND_H
+    if (RUNNING_ON_VALGRIND) {
+        /* This malformed message does not crash the server, but keeps waiting
+         * for more data as announced in the payloiad length so the opened FD on
+         * the server side is leaking when the server terminates.
+         * Given that the custom sftp server could store anything into the
+         * handles, it should take care of cleaning up the outstanding handles,
+         * but this is something to solve in the future. Now just skipping the
+         * test.
+         */
+        skip();
+    }
+#endif /* HAVE_VALGRIND_VALGRIND_H */
+
+    assert_non_null(tss);
+
+    s = tss->state;
+    assert_non_null(s);
+
+    tsftp = s->ssh.tsftp;
+    assert_non_null(tsftp);
+
+    sftp = tsftp->sftp;
+    assert_non_null(sftp);
+
+    /* Open a file for writing */
+    snprintf(name, sizeof(name), "%s/file", tsftp->testdir);
+    handle = sftp_open(sftp, name, O_WRONLY | O_CREAT, 0700);
+    assert_non_null(handle);
+
+    /* Craft an malicious SFTP packet trying to write to the file with
+     * payload_length overrun */
+    buffer = ssh_buffer_new();
+    id = sftp_get_new_id(sftp);
+    assert_non_null(buffer);
+
+    rc = ssh_buffer_pack(buffer,
+                         "dbdSqd",
+                         bad_payload_len,
+                         SSH_FXP_WRITE,
+                         id,
+                         handle->handle,
+                         (uint64_t)0,
+                         (uint32_t)0);
+    assert_int_equal(rc, SSH_OK);
+    rc = ssh_channel_write(sftp->channel,
+                           ssh_buffer_get(buffer),
+                           ssh_buffer_get_len(buffer));
+    assert_int_equal(rc, 29);
+    SSH_BUFFER_FREE(buffer);
+
+    /* We do not get answer for this malformed packet -- just kill the
+     * connection */
+    ssh_string_free(handle->handle);
+    free(handle);
+}
 
 int torture_run_tests(void) {
     int rc;
@@ -1131,6 +1290,12 @@ int torture_run_tests(void) {
         cmocka_unit_test_setup_teardown(torture_server_sftp_handles_exhaustion,
                                         session_setup_sftp,
                                         session_teardown),
+        cmocka_unit_test_setup_teardown(torture_server_sftp_handle_overrun,
+                                        session_setup_sftp,
+                                        session_teardown),
+        cmocka_unit_test_setup_teardown(torture_server_sftp_payload_overrun,
+                                        session_setup_sftp,
+                                        session_teardown),
     };
 
     ssh_init();
