@@ -438,7 +438,7 @@ static void torture_pki_ecdsa_import_cert_file(void **state)
     SSH_KEY_FREE(pubkey);
 
     /* Generate different key and try to assign it this certificate */
-    rc = ssh_pki_generate(test_state->type, 256, &privkey);
+    rc = ssh_pki_generate_key(test_state->type, NULL, &privkey);
     assert_return_code(rc, errno);
     assert_non_null(privkey);
     rc = ssh_pki_export_privkey_to_pubkey(privkey, &pubkey);
@@ -1094,12 +1094,14 @@ int torture_run_tests(void) {
         cmocka_unit_test_setup_teardown(torture_pki_ecdsa_import_pubkey_file,
                                         setup_ecdsa_key_521,
                                         teardown),
-        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_import_pubkey_from_openssh_privkey,
-                                        setup_openssh_ecdsa_key_256,
-                                        teardown),
-        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_import_pubkey_from_openssh_privkey,
-                                        setup_openssh_ecdsa_key_384,
-                                        teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_import_pubkey_from_openssh_privkey,
+            setup_openssh_ecdsa_key_256,
+            teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_import_pubkey_from_openssh_privkey,
+            setup_openssh_ecdsa_key_384,
+            teardown),
         cmocka_unit_test_setup_teardown(torture_pki_ecdsa_import_pubkey_file,
                                         setup_openssh_ecdsa_key_521,
                                         teardown),
@@ -1115,24 +1117,30 @@ int torture_run_tests(void) {
             torture_pki_ecdsa_import_export_privkey_base64_default,
             setup_ecdsa_key_521,
             teardown),
-        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_import_privkey_base64_comment,
-                                        setup_ecdsa_key_256,
-                                        teardown),
-        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_import_privkey_base64_comment,
-                                        setup_ecdsa_key_384,
-                                        teardown),
-        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_import_privkey_base64_comment,
-                                        setup_ecdsa_key_521,
-                                        teardown),
-        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_import_privkey_base64_whitespace,
-                                        setup_ecdsa_key_521,
-                                        teardown),
-        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_import_privkey_base64_whitespace,
-                                        setup_ecdsa_key_521,
-                                        teardown),
-        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_import_privkey_base64_whitespace,
-                                        setup_ecdsa_key_521,
-                                        teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_import_privkey_base64_comment,
+            setup_ecdsa_key_256,
+            teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_import_privkey_base64_comment,
+            setup_ecdsa_key_384,
+            teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_import_privkey_base64_comment,
+            setup_ecdsa_key_521,
+            teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_import_privkey_base64_whitespace,
+            setup_ecdsa_key_521,
+            teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_import_privkey_base64_whitespace,
+            setup_ecdsa_key_521,
+            teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_import_privkey_base64_whitespace,
+            setup_ecdsa_key_521,
+            teardown),
         cmocka_unit_test_setup_teardown(
             torture_pki_ecdsa_import_export_privkey_base64_default,
             setup_openssh_ecdsa_key_256,
@@ -1145,15 +1153,18 @@ int torture_run_tests(void) {
             torture_pki_ecdsa_import_export_privkey_base64_default,
             setup_openssh_ecdsa_key_521,
             teardown),
-        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_publickey_from_privatekey,
-                                        setup_ecdsa_key_256,
-                                        teardown),
-        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_publickey_from_privatekey,
-                                        setup_ecdsa_key_384,
-                                        teardown),
-        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_publickey_from_privatekey,
-                                        setup_ecdsa_key_521,
-                                        teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_publickey_from_privatekey,
+            setup_ecdsa_key_256,
+            teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_publickey_from_privatekey,
+            setup_ecdsa_key_384,
+            teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_publickey_from_privatekey,
+            setup_ecdsa_key_521,
+            teardown),
         cmocka_unit_test_setup_teardown(torture_pki_ecdsa_import_cert_file,
                                         setup_ecdsa_key_256,
                                         teardown),
@@ -1181,15 +1192,18 @@ int torture_run_tests(void) {
         cmocka_unit_test_setup_teardown(torture_pki_ecdsa_publickey_base64,
                                         setup_ecdsa_key_521,
                                         teardown),
-        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_generate_pubkey_from_privkey,
-                                        setup_ecdsa_key_256,
-                                        teardown),
-        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_generate_pubkey_from_privkey,
-                                        setup_ecdsa_key_384,
-                                        teardown),
-        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_generate_pubkey_from_privkey,
-                                        setup_ecdsa_key_521,
-                                        teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_generate_pubkey_from_privkey,
+            setup_ecdsa_key_256,
+            teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_generate_pubkey_from_privkey,
+            setup_ecdsa_key_384,
+            teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_generate_pubkey_from_privkey,
+            setup_ecdsa_key_521,
+            teardown),
         cmocka_unit_test_setup_teardown(torture_pki_ecdsa_duplicate_key,
                                         setup_ecdsa_key_256,
                                         teardown),
